from PySide6.QtCore import Qt, Slot
from PySide6.QtWidgets import (
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QMessageBox,
    QSplitter,
    QTabWidget,
    QWidget,
)

from bn_modeller.models import (
    CorrelationSQLProxyModel,
    FilterPairTableSQLProxyModel,
    PairTableSQLProxyModel,
)
from bn_modeller.models.feature_sqltable_model import (
    FeatureSqlTableModel,
    PersistanceCheckableFeatureListProxyModel,
)
from bn_modeller.widgets import DependencySetupTableView, SelectableListView
from bn_modeller.widgets.bn_visualization_view import BayesianNetView
from bn_modeller.widgets.vertical_label import QVertivalLabel


class BayesianNetworkPageWidget(QWidget):
    def __init__(self, parent: QWidget | None = None, f=Qt.WindowType()):
        super().__init__(parent, f)
        self._init_ui()

    def _init_ui(self):
        self.mainLayout = QHBoxLayout(self)

        self.tabWidget = QTabWidget()

        # Dependency tab

        # Feature selection
        self.dependencyTabWidget = QSplitter()
        self.featureSelectorView = SelectableListView()
        self.featureSelectorView.setStatusTip(
            self.tr("Select features for the Bayesian Network.")
        )
        self.dependencyTabWidget.addWidget(self.featureSelectorView)
        self.dependencyTabWidget.setStretchFactor(0, 1)

        # Dependency Table
        depTableWidget = QWidget()
        depTableLayout = QGridLayout()

        self._depTable = DependencySetupTableView()
        depTableLayout.addWidget(self._depTable, 1, 1)

        independentLabel = QVertivalLabel("Independent")
        depTableLayout.addWidget(independentLabel, 1, 0, Qt.AlignmentFlag.AlignCenter)

        dependentLabel = QLabel("Dependent")
        depTableLayout.addWidget(dependentLabel, 0, 1, Qt.AlignmentFlag.AlignCenter)

        explanationLabel = QLabel(
            self.tr(
                "Here you can setup expert-defined dependencies between features, choosen in the left side of the window.\n"
                "In the table you can see Spearman correlation coefficients between features as well as partial correlation coefficients.\n"
                "The background color of the table cells indicates the strength of the correlation.\n\n"
                "To visualize the Bayesian Network, open the 'Visualization' tab.\n\n"
                "Please note, that these dependencies are not learned from the data. "
                "They are defined by experts and used to guide the learning process.\n\n"
                "Hint: You can select entire rows or columns by clicking using right mouse button on the row or column header."
            )
        )
        depTableLayout.addWidget(
            explanationLabel, 2, 0, 1, 2, Qt.AlignmentFlag.AlignLeft
        )

        depTableWidget.setLayout(depTableLayout)
        self.dependencyTabWidget.addWidget(depTableWidget)
        self.dependencyTabWidget.setStretchFactor(1, 2)

        self.tabWidget.addTab(self.dependencyTabWidget, self.tr("Dependency"))

        # Visualization Tab
        self.visualizationTabWidget = BayesianNetView()

        self.tabWidget.addTab(self.visualizationTabWidget, self.tr("Visulization"))

        # Finalization
        self.mainLayout.addWidget(self.tabWidget)
        self.setLayout(self.mainLayout)

    def setModels(self, pairTableSQLProxyModel: PairTableSQLProxyModel):
        # TODO: т.к. при реализации обсчета байесовских сетей нужно использовать
        # self._pairTableSQLProxyModel, вероятно, её нужно будет вытащить наружу
        # или сделать рассчеты дочерним объектом этой страницы

        self._featureCheckableSortFilterProxyModel = (
            PersistanceCheckableFeatureListProxyModel()
        )
        self._featureCheckableSortFilterProxyModel.setSourceModel(
            pairTableSQLProxyModel.getFeatureSqlTableModel()
        )

        self.featureSelectorView.setModel(self._featureCheckableSortFilterProxyModel)
        self.featureSelectorView.setModelColumn(
            pairTableSQLProxyModel.getFeatureSqlTableModel().fieldIndex(
                pairTableSQLProxyModel.getFeatureSqlTableModel().column_name
            )
        )

        filterPairTableSQLProxyModel: FilterPairTableSQLProxyModel = (
            FilterPairTableSQLProxyModel()
        )
        filterPairTableSQLProxyModel.setSourceModel(pairTableSQLProxyModel)
        filterPairTableSQLProxyModel.setFilterModel(
            self._featureCheckableSortFilterProxyModel,
            pairTableSQLProxyModel.getFeatureSqlTableModel().fieldIndex(
                FeatureSqlTableModel.column_id
            ),
        )

        self._correlationModel = CorrelationSQLProxyModel(filterPairTableSQLProxyModel)
        self._correlationModel.partialCorrGeneralError.connect(
            self.onPartialCorrError, type=Qt.ConnectionType.QueuedConnection
        )

        self._depTable.setModel(self._correlationModel)

        self.visualizationTabWidget.setModels(self._correlationModel)

    @Slot()
    def onPartialCorrError(self):
        res = QMessageBox.critical(
            self,
            self.tr("Partial Correlation Error"),
            self.tr(
                "An error occurred while calculating the partial correlation. \n\n"
                "Highly likely, it is caused by a missing value in the data set.\n"
                "To resolve this issue, you can try to disable features with all NAN values in the correlation matrix or select additional features that might help fill in the gaps."
            ),
            QMessageBox.StandardButton.Ok,
            QMessageBox.StandardButton.Ok,
        )



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QFrame, QWidget


class QSeparator(QFrame):
    """A separator widget.
    A separator is a line  that are used to separate
    different parts of the user interface.

    Usage example:

    >>> separator = QSeparator()
    >>> layout.addWidget(separator)

    """

    def __init__(
        self, parent: QWidget | None = None, f: Qt.WindowType = Qt.WindowType()
    ):
        super().__init__(parent, f)
        self.setFrameShape(QFrame.Shape.HLine)
        self.setFrameShadow(QFrame.Shadow.Sunken)



import os
import sys

import numpy as np
import pandas as pd
from button_page import ButtonWindow
from input_output import IOWindow
from jointgrid import jointgridWindow
from link_table import LinkTabWindow
from PySide6.QtCore import Slot
from PySide6.QtWidgets import (
    QApplication,
    QMainWindow,
    QTabWidget,
    QVBoxLayout,
    QWidget,
)

from bn_modeller.bayesian_nets.utils import check_non_select_table, find_zero_columns


class App(QMainWindow):

    def __init__(self):
        super().__init__()
        self.title = "Bayes model"
        self.left = 0
        self.top = 0
        self.width = 1500
        self.height = 1000
        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)

        self.table_widget = MyTableWidget(self)
        self.setCentralWidget(self.table_widget)

        self.show()

    def closeEvent(self, event):
        for window in QApplication.topLevelWidgets():
            window.close()


class MyTableWidget(QWidget):

    def __init__(self, parent):
        super().__init__(parent)
        self.layout = QVBoxLayout(self)

        self.pathIOFile = "data/io_table.csv"
        self.pathLinkFile = "data/link_table.xlsx"
        self.pathInputFile = "data/data_2.csv"

        self.input_feature = None
        self.output_feature = None
        self.lenCheck = len(pd.read_csv(self.pathInputFile, index_col=0).columns)
        self.stateIO = self.initStateIO()
        self.stateLink = self.initStateLinkTable()
        self.linkTable = None
        self.stateJointplot = None

        self.setWindowTitle("Bayes inference")

        # Initialize tab screen
        self.tabs = QTabWidget()
        self.tabs.resize(300, 200)

        # Initialize Widgets for tabs
        self.ioTab = IOWindow(
            self, input_df=self.getInitialDataframe(), state=self.getStateIO()
        )
        self.update()

        self.linkTab = LinkTabWindow(
            self, input_df=self.getDataFrame(), state=self.stateLink
        )

        self.buttonTab = ButtonWindow(self, pd.DataFrame(), pd.DataFrame(), 1)
        self.buttonTab.acycle_button.clicked.connect(self.updateLinkTableFromAcyclic)

        self.jointgridTab = jointgridWindow(
            self,
            data=self.getInitialDataframe(),
            predicted_data=self.buttonTab.getPrediction(),
        )

        self.tabs.resize(300, 200)

        # Add tabs
        self.tabs.addTab(self.ioTab, "Input-Output")
        self.tabs.addTab(self.linkTab, "Links")
        self.tabs.addTab(self.buttonTab, "Calculation")
        self.tabs.addTab(self.jointgridTab, "Plot")
        self.tabs.currentChanged.connect(self.on_click)  # changed!
        self.ioTab.block_signal.connect(self.blockButtonTab)

        # Add tabs to widget
        self.layout.addWidget(self.tabs)
        self.setLayout(self.layout)

    @Slot()
    def on_click(self):
        """
        action between tabs
        """
        index = self.tabs.currentIndex()
        if index == 0:
            # print("Switched to Tab 1")
            return 1

        elif index == 1:
            self.blockButtonTab()

        elif index == 2:

            self.linkTab.save_clicked()
            self.updateStateLink()
            self.updateLinkTable()

            # проверить выделены ли хоть какие-то связи

            if check_non_select_table(self.linkTable):
                self.buttonTab.acycle_button.setEnabled(False)
                self.buttonTab.rankCorrButton.setEnabled(False)
                self.buttonTab.calcInferenceButton.setEnabled(False)
            else:
                self.buttonTab.acycle_button.setEnabled(True)
            self.tmp()

        elif index == 3:
            self.jointgridTab.removeTableWidget()
            self.jointgridTab.updateData(self.buttonTab.getPrediction())

    @Slot()
    def blockButtonTab(self):
        self.ioTab.save_clicked()
        self.ioTab.saveState()
        self.update()
        self.updateStateIO()

        if self.stateIO is None:
            self.tabs.setTabEnabled(2, False)
        else:
            # the value of each tick is 2
            # in order for the calculations tab to be opened, you need to click at least 2 "in" and 1 "out" checkboxes
            if (
                sum([i[0].value for i in self.stateIO]) >= 4
                and sum([i[1].value for i in self.stateIO]) >= 2
            ):
                self.tabs.setTabEnabled(2, True)
            else:
                self.tabs.setTabEnabled(2, False)

    @Slot()
    def update(self):
        self.updateFeaturesList(
            self.ioTab.getInputFeature(), self.ioTab.getOutputFeature()
        )

    def getInitialDataframe(self):
        data_input = pd.read_csv(self.pathInputFile, index_col=0)
        # add_quotes = lambda x: f'"{x}"'
        # data_input.columns = list(map(add_quotes, data_input.columns))
        data_input = data_input.replace("[^0-9]+", np.nan, regex=True)
        data_input = data_input.astype(float)
        nan_columns = find_zero_columns(data_input)
        data_input = data_input.drop(nan_columns, axis=1)
        # if not self.input_feature is None:
        #     data_input = data_input.loc[:, self.input_feature + self.output_feature]
        return data_input

    def updateFeaturesList(self, input_features, output_features):
        self.input_feature = input_features
        self.output_feature = output_features

    def getDataFrame(self):
        df = self.getInitialDataframe()
        df = df.loc[:, self.input_feature + self.output_feature]
        return df

    def updateLinkTable(self):
        self.linkTable = self.linkTab.getDataFrame()

    def updateLinkTableFromAcyclic(self):
        if self.buttonTab.updLinkTable is not None:
            self.linkTable = self.buttonTab.getNewLinkTab()
        self.stateLink = self.linkTable.values
        self.linkTab.removeTableWidget()
        self.linkTab.updateInput(self.getDataFrame(), self.getStateLink())

    def getLinkTable(self):
        return self.linkTable

    def getStateIO(self):
        return self.stateIO

    def getStateJointplot(self):
        return self.stateJointplot

    def getStateLink(self):
        return self.stateLink

    def updateStateIO(self):
        self.stateIO = self.ioTab.getState()
        if set(self.linkTab.getColumnNames()) != set(
            self.input_feature + self.output_feature
        ):
            self.linkTab.removeTableWidget()
            self.linkTab.updateInput(self.getDataFrame(), self.getStateLink())
        self.saveStateIO()

    def saveStateIO(self):
        df = pd.DataFrame(
            columns=["input", "output"],
            index=["Select ALL"] + list(self.getInitialDataframe().columns),
            data=self.stateIO,
        )
        df.to_csv(self.pathIOFile)

    def initStateIO(self):
        if os.path.exists(self.pathIOFile):
            df = pd.read_csv(self.pathIOFile, index_col=0)
            if self.lenCheck != len(df.index):
                return None
            return df.values
        else:
            return None

    def initStateLinkTable(self):
        if os.path.exists(self.pathLinkFile):
            df = pd.read_excel(self.pathLinkFile, index_col=0)
            # print(self.stateIO.)
            # # df.insert(loc=0, column='Select All', value=[0]*len(df))
            # if self.lenCheck != len(df.columns):
            #     return None
            return df.values
        else:
            return None

    def updateStateLink(self):
        self.stateLink = self.linkTab.state

    def tmp(self):
        self.buttonTab.updateDataFrame(self.getDataFrame())
        self.buttonTab.updateLinkTable(self.getLinkTable())
        self.buttonTab.updateLenInputFeature(len(self.input_feature))
        self.buttonTab.updateMatrix()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    ex = App()
    sys.exit(app.exec())



from PySide6.QtCore import QAbstractItemModel, Qt, Signal, Slot
from PySide6.QtGui import QAction
from PySide6.QtWidgets import QAbstractItemView, QHBoxLayout, QListView, QMenu, QWidget

from bn_modeller.models.sample_sqltable_model import SampleSqlTableModel
from bn_modeller.widgets.all_samples_view import AllSamplesView


class SelectableListView(QListView):
    def __init__(self, parent: QWidget | None = None):
        super().__init__(parent)
        self.setSelectionMode(QAbstractItemView.SelectionMode.MultiSelection)
        self.setSelectionRectVisible(True)
        self.initContextMenu()

    def initContextMenu(self):
        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.customContextMenuRequested.connect(self.showContextMenu)

        self.menu = QMenu(self)

        selectBySelection = QAction(self.tr("Select"), self)
        selectBySelection.triggered.connect(self.setCheckStateBySelection)
        self.menu.addAction(selectBySelection)

        unselectBySelection = QAction(self.tr("Unselect"), self)
        unselectBySelection.triggered.connect(self.removeCheckStateBySelection)
        self.menu.addAction(unselectBySelection)

        self.menu.addSeparator()

        selectAllAction = QAction(self.tr("Select All"), self)
        selectAllAction.triggered.connect(self.setCheckStateAll)
        self.menu.addAction(selectAllAction)

        unselectAllAction = QAction(self.tr("Unselect All"), self)
        unselectAllAction.triggered.connect(self.removeCheckStateAll)
        self.menu.addAction(unselectAllAction)

    @Slot()
    def showContextMenu(self, pos):
        self.menu.popup(self.mapToGlobal(pos))

    @Slot()
    def setCheckStateBySelection(self):
        source = self.sender()
        self._setCheckStateForSelectionBySender(source, True)

    @Slot()
    def removeCheckStateBySelection(self):
        source = self.sender()
        self._setCheckStateForSelectionBySender(source, False)

    @Slot()
    def setCheckStateAll(self):
        source = self.sender()
        self._setAllCheckStateBySender(source, True)

    @Slot()
    def removeCheckStateAll(self):
        source = self.sender()
        self._setAllCheckStateBySender(source, False)

    def _setAllCheckStateBySender(self, source, checkState: bool):
        if isinstance(source, QAction):
            for r in range(self.model().rowCount()):
                self.model().setData(
                    self.model().index(r, 0),
                    checkState,
                    role=Qt.ItemDataRole.CheckStateRole,
                )

    def _setCheckStateForSelectionBySender(self, source, checkState: bool):
        if isinstance(source, QAction):
            for index in self.selectedIndexes():
                self.model().setData(
                    index,
                    checkState,
                    role=Qt.ItemDataRole.CheckStateRole,
                )



import pandas as pd
from PySide6.QtCore import QAbstractItemModel, Qt


def tablemodel_to_dataframe(
    model: QAbstractItemModel, role: int, postprocess: callable = lambda x: x
) -> pd.DataFrame:
    """Converts a QAbstractItemModel to a pandas DataFrame.

    Args:
        model (QAbstractItemModel): Source QAbstractItemModel.
        role (int): Role to extract from the model.
        postprocess (callable, optional): Function to apply to each element. Defaults to lambda x: x.
    Returns:
        pd.DataFrame: Pandas DataFrame containing the model's data.
    """
    data_pd = pd.DataFrame(
        columns=[
            model.headerData(c, Qt.Orientation.Horizontal)
            for c in range(model.columnCount())
        ]
    )
    for r in range(model.rowCount()):
        new_row = pd.Series(
            {
                model.headerData(c, Qt.Orientation.Horizontal): postprocess(
                    model.index(r, c).data(role=role)
                )
                for c in range(model.columnCount())
            }
        )
        data_pd.loc[model.headerData(r, Qt.Orientation.Vertical)] = new_row

    return data_pd



from .checkable_sort_filter_proxy_model import CheckableSortFilterProxyModel
from .dependency_sqltable_model import (
    CorrelationSQLProxyModel,
    DependencyManyToManySqlTableModel,
    FilterPairTableSQLProxyModel,
    PairTableSQLProxyModel,
)
from .relational_sort_filter_proxy_model import RelationalSortFilterProxyModel
from .sample_sqltable_model import SampleSqlTableModel



from PySide6.QtCore import QAbstractItemModel
from PySide6.QtWidgets import QTableView, QWidget

from bn_modeller.models.sample_sqltable_model import SampleSqlTableModel


class AllSamplesView(QTableView):
    def __init__(self, parent: QWidget | None = None):
        super().__init__(parent)

    def setModel(self, model: SampleSqlTableModel):
        super().setModel(model)



from PySide6.QtCore import QPoint, QRect, QSize, Qt
from PySide6.QtGui import QPainter
from PySide6.QtWidgets import QLabel, QWidget


class QVertivalLabel(QLabel):
    def __init__(
        self,
        text: str,
        parent: QWidget | None = None,
        f: Qt.WindowType = Qt.WindowType(),
    ):
        super().__init__(text, parent, f)
        # self.setSizePolicy(QSizePolicy.Policy.Minimum,
        #                    QSizePolicy.Policy.Expanding)

    def paintEvent(self, arg__1):
        painter = QPainter(self)
        painter.translate(0, self.sizeHint().height())
        painter.rotate(270)
        painter.drawText(
            QRect(QPoint(0, 0), super().sizeHint()),
            Qt.AlignmentFlag.AlignCenter,
            self.text(),
        )

    def minimumSizeHint(self):
        s = super().minimumSizeHint()
        return QSize(s.height(), s.width())

    def sizeHint(self):
        s = super().sizeHint()
        return QSize(s.height(), s.width())



import pandas as pd
from PySide6 import QtCore, QtWidgets


class IOWindow(QtWidgets.QWidget):
    block_signal = QtCore.Signal()

    def __init__(self, parent=None, input_df=None, state=None):
        super(IOWindow, self).__init__(parent)
        self.input_table = input_df
        col_name = self.input_table.columns

        self.df_input_output = pd.DataFrame(columns=["input", "output"], index=col_name)
        col_name = col_name.insert(0, "Select ALL")

        self.tableWidget = QtWidgets.QTableWidget(len(col_name), 2)
        lay = QtWidgets.QVBoxLayout(self)
        lay.addWidget(self.tableWidget)

        self.tableWidget.setHorizontalHeaderLabels(["input", "output"])
        self.tableWidget.setVerticalHeaderLabels(col_name)
        self.state = self.updateState(state) if state is not None else self.initState()

        for i in range(self.tableWidget.rowCount()):
            for j in range(2):
                item = QtWidgets.QTableWidgetItem()
                item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)
                item.setCheckState(self.state[i][j])
                self.tableWidget.setItem(i, j, item)

        self.tableWidget.cellChanged.connect(self.select_all_clicked_by_columns)
        self.tableWidget.cellChanged.connect(self.turnOffClicked)
        self.tableWidget.cellChanged.connect(self.block_signal)

    def send_block_signal(self):
        self.block_signal.emit()

    def select_all_clicked_by_columns(self, row, column):
        if row != 0:
            return 0
        state = self.tableWidget.item(0, column).checkState()
        for i in range(self.tableWidget.rowCount()):
            item = self.tableWidget.item(i, column)
            item.setCheckState(QtCore.Qt.Checked if state else QtCore.Qt.Unchecked)

    def turnOffClicked(self, row, column):
        state = self.tableWidget.item(row, column).checkState()

        all_columns = list(range(self.tableWidget.columnCount()))
        all_columns.remove(column)

        for i in all_columns:
            item = self.tableWidget.item(row, i)
            if state == QtCore.Qt.Checked:
                item.setCheckState(QtCore.Qt.Unchecked)
                # item.setCheckState(QtCore.Qt.Checked)
            # else:
            # item.setCheckState(QtCore.Qt.Unchecked)
            # item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)

    @QtCore.Slot()
    def save_clicked(self):
        items = []
        for i in range(1, self.tableWidget.rowCount()):
            it = []
            for j in range(0, self.tableWidget.columnCount()):
                item = self.tableWidget.item(i, j)
                it.append(item.checkState())
            items.append(it)

        self.df_input_output.loc[:, :] = items

    def getDataframe(self):
        # Use this method to retrieve the dataframe
        return self.df_input_output

    def setDataframe(self, df):
        self.input_table = df

    def getInputFeature(self):
        return list(
            self.df_input_output[
                self.df_input_output["input"].map(
                    lambda x: x.value if isinstance(x, QtCore.Qt.CheckState) else x
                )
                > 0
            ].index
        )

    def getOutputFeature(self):
        return list(
            self.df_input_output[
                self.df_input_output["output"].map(
                    lambda x: x.value if isinstance(x, QtCore.Qt.CheckState) else x
                )
                > 0
            ].index
        )

    def saveState(self):
        self.state = []
        for i in range(self.tableWidget.rowCount()):
            it = []
            for j in range(self.tableWidget.columnCount()):
                item = self.tableWidget.item(i, j)
                it.append(item.checkState())
            self.state.append(it)

    def initState(self):
        return [[QtCore.Qt.Unchecked] * 2 for _ in range(self.tableWidget.rowCount())]

    def updateState(self, input_vector):
        new_vector = [
            [QtCore.Qt.Unchecked] * 2 for _ in range(self.tableWidget.rowCount())
        ]
        for i in range(len(input_vector)):
            for j in range(len(input_vector[i])):
                new_vector[i][j] = (
                    QtCore.Qt.Checked if input_vector[i][j] > 0 else QtCore.Qt.Unchecked
                )
        return new_vector

    def getState(self):
        return self.state



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QSplitter, QVBoxLayout, QWidget

from bn_modeller.models import (
    CheckableSortFilterProxyModel,
    RelationalSortFilterProxyModel,
)
from bn_modeller.models.feature_sqltable_model import FeatureSqlTableModel
from bn_modeller.models.sample_sqltable_model import SampleSqlTableModel
from bn_modeller.widgets.all_samples_view import AllSamplesView
from bn_modeller.widgets.plots import PairplotView
from bn_modeller.widgets.selectable_list_view import SelectableListView


class DatabasePageWidget(QSplitter):
    def __init__(self, parent: QWidget | None = None, f=Qt.WindowType()):
        super().__init__(parent, f)
        self._init_ui()

    def _init_ui(self):
        self.setOrientation(Qt.Orientation.Horizontal)

        # Left side widget with feature selector view
        leftSideWidget = QWidget()
        leftSideWidgetLayout = QVBoxLayout(leftSideWidget)

        self.featureSelectorView = SelectableListView()
        self.featureSelectorView.setStatusTip(
            self.tr("Select features for visualization.")
        )
        leftSideWidgetLayout.addWidget(self.featureSelectorView)

        featureSelectorExplanationLabel = QLabel(
            self.tr(
                "Here you can select features to evaluate and visualize correlations between them in the pairplot view."
            )
        )
        featureSelectorExplanationLabel.setWordWrap(True)
        leftSideWidgetLayout.addWidget(featureSelectorExplanationLabel)

        self.addWidget(leftSideWidget)

        # Right side widget with pairplot view

        rightSideWidget = QWidget()
        rightSideWidgetLayout = QVBoxLayout(rightSideWidget)

        self.pairPlorView = PairplotView()
        rightSideWidgetLayout.addWidget(self.pairPlorView)

        pairplotExplanationLabel = QLabel(
            self.tr(
                "Here you can see a pairplot of selected features. This allows you to visualize correlations between selected features. The number in the top of each subplot represents the Spearman correlation coefficient between the two features."
            )
        )
        pairplotExplanationLabel.setWordWrap(True)
        rightSideWidgetLayout.addWidget(pairplotExplanationLabel)

        self.addWidget(rightSideWidget)

        # self.databaseView = AllSamplesView()
        # self.addWidget(self.databaseView)

    def setModels(
        self,
        featureSqlTableModel: FeatureSqlTableModel,
        sampleSqlTableModel: SampleSqlTableModel,
    ):
        self._sampleSqlTableModel = sampleSqlTableModel
        self._featureSqlTableModel = featureSqlTableModel

        self._featureCheckableSortFilterProxyModel = CheckableSortFilterProxyModel()
        self._featureCheckableSortFilterProxyModel.setSourceModel(
            self._featureSqlTableModel
        )
        self.featureSelectorView.setModel(self._featureCheckableSortFilterProxyModel)
        self.featureSelectorView.setModelColumn(
            featureSqlTableModel.fieldIndex(featureSqlTableModel.column_name)
        )

        self.visualisationProxyModel = RelationalSortFilterProxyModel()
        self.visualisationProxyModel.setSourceModel(self._sampleSqlTableModel)
        self.visualisationProxyModel.setFilterModel(
            self._featureCheckableSortFilterProxyModel,
            self._featureSqlTableModel.fieldIndex(FeatureSqlTableModel.column_id),
        )
        self.visualisationProxyModel.setFilterKeyColumn(
            self._sampleSqlTableModel.fieldIndex(SampleSqlTableModel.column_feature_id)
        )

        # self.databaseView.setModel(self.visualisationProxyModel)
        self.pairPlorView.setModel(
            self.visualisationProxyModel,
            self._featureSqlTableModel.fieldIndex(FeatureSqlTableModel.column_name),
            self._sampleSqlTableModel.fieldIndex(SampleSqlTableModel.column_sample_id),
            self._sampleSqlTableModel.fieldIndex(SampleSqlTableModel.column_feature_id),
            self._sampleSqlTableModel.fieldIndex(SampleSqlTableModel.column_value),
        )



import matplotlib
import numpy as np
import pandas as pd
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg, NavigationToolbar2QT
from matplotlib.figure import Figure
from PySide6.QtCore import (
    QAbstractItemModel,
    QModelIndex,
    QObject,
    QPersistentModelIndex,
    Qt,
    Signal,
    Slot,
)
from PySide6.QtWidgets import QFrame, QVBoxLayout, QWidget
from scipy import stats

from bn_modeller.models import RelationalSortFilterProxyModel
from bn_modeller.models.checkable_sort_filter_proxy_model import (
    CheckableSortFilterProxyModel,
)

for be in ["pdf", "pgf", "ps", "svg"]:
    matplotlib.use(be)
matplotlib.use("qt5agg")


class PairplotMplCanvas(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=12, height=12, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        super().__init__(self.fig)
        self.axes = None

    def update_plot(self, data_pd):
        if len(data_pd) == 0:
            return

        labels = data_pd.columns
        num_vars = len(labels)

        self.fig.clear()

        # Create new subplots
        if num_vars == 1:
            self.axes = self.fig.add_subplot(111)
        else:
            self.axes = self.fig.subplots(num_vars, num_vars)

        for i in range(num_vars):
            for j in range(num_vars):
                if num_vars == 1:
                    ax = self.axes
                else:
                    ax = self.axes[i, j]

                if i == j:
                    ax.hist(data_pd.iloc[:, i])
                    if j == 0:
                        ax.set_ylabel(labels[i])
                    if i == num_vars - 1:
                        ax.set_xlabel(labels[j])
                else:
                    ax.scatter(data_pd.iloc[:, j], data_pd.iloc[:, i], alpha=0.6)
                    ax.set_xlabel("" if i != num_vars - 1 else labels[j])
                    ax.set_ylabel("" if j != 0 else labels[i])

                    # Add correlation
                    df1, df2 = data_pd.iloc[:, i], data_pd.iloc[:, j]
                    # df1_cleaned, df2_cleaned = df1.align(df2, join='inner')
                    df1_cleaned, df2_cleaned = df1.dropna(), df2.dropna()

                    # Убираем те строки, в которых один из столбцов имеет NaN
                    df1_cleaned, df2_cleaned = df1_cleaned.align(
                        df2_cleaned, join="inner"
                    )

                    corr = stats.spearmanr(df1_cleaned, df2_cleaned).statistic
                    ax.annotate(
                        f"{corr:.2f}",
                        xy=(0.5, 0.9),
                        xycoords="axes fraction",
                        ha="center",
                        fontsize=10,
                        color="red",
                    )

        self.fig.tight_layout(pad=3.0)
        self.fig.subplots_adjust(hspace=0.3, wspace=0.3)
        self.draw()


class PairplotView(QFrame):
    def __init__(self, parent: QWidget = None, f=Qt.WindowType()):
        super().__init__(parent, f)
        self._model: RelationalSortFilterProxyModel = None
        self.init_ui()

    def init_ui(self):
        self.mainLayout = QVBoxLayout()
        self.mplCanvas = PairplotMplCanvas(self)
        self.toolbar = NavigationToolbar2QT(self.mplCanvas, self)
        self.mainLayout.addWidget(self.toolbar)
        self.mainLayout.addWidget(self.mplCanvas)
        self.setLayout(self.mainLayout)

    def setModel(
        self,
        model: RelationalSortFilterProxyModel,
        hue_names_col: int,
        sample_id_col: int,
        hue_id_col: int,
        value_col: int,
    ):
        if self._model is not None:
            self._model.filterInvalidated.disconnect(self.updateVisualization)
        self._model = model
        self._model.filterInvalidated.connect(self.updateVisualization)

        self._value_col = value_col
        self._sample_id_col = sample_id_col
        self._hue_id_col = hue_id_col
        self._hue_names_col = hue_names_col

    @Slot()
    def updateVisualization(self):
        filterModel = self._model.filterModel()
        filter_labels = {
            filterModel.data(
                filterModel.index(rowIdx, self._model.filterValueColumn())
            ): filterModel.data(filterModel.index(rowIdx, self._hue_names_col))
            for rowIdx in range(filterModel.rowCount())
        }

        data_dict = {"sample": [], "value": [], "label": []}
        for rowIdx in range(self._model.rowCount()):
            data_dict["sample"].append(
                self._model.data(self._model.index(rowIdx, self._sample_id_col))
            )
            data_dict["label"].append(
                filter_labels[
                    self._model.data(self._model.index(rowIdx, self._hue_id_col))
                ]
            )
            data_dict["value"].append(
                self._model.data(self._model.index(rowIdx, self._value_col))
            )

        data_pd = pd.DataFrame.from_dict(data_dict).pivot(
            index="sample", columns="label", values="value"
        )

        if len(data_pd) == 0:
            return

        filtered_order = [
            col for col in filter_labels.values() if col in data_pd.columns
        ]
        data_pd = data_pd[filtered_order]

        self.mplCanvas.update_plot(data_pd)



import itertools as it

import networkx as nx
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg, NavigationToolbar2QT
from matplotlib.figure import Figure
from PySide6.QtCore import Qt, Signal, Slot
from PySide6.QtWidgets import (
    QCheckBox,
    QComboBox,
    QHBoxLayout,
    QLabel,
    QSplitter,
    QVBoxLayout,
    QWidget,
)

from bn_modeller.bayesian_nets.graph_preparation import GraphPreparation
from bn_modeller.models import CorrelationSQLProxyModel, PairTableSQLProxyModel
from bn_modeller.utils.model_adapters import tablemodel_to_dataframe
from bn_modeller.widgets.extended_slider_widget import ExtendedSliderWidget


class BayesianNetCanvas(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=12, height=12, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        super().__init__(self.fig)
        self.bn_ax = self.fig.add_subplot(1, 1, 1)

    def update_plot(self, graph):
        self.bn_ax.clear()
        # self.bn_ax = self.fig.add_subplot(1, 1, 1)

        elarge = [(u, v) for (u, v, d) in graph.edges(data=True) if d["weight"] > 0.5]
        esmall = [(u, v) for (u, v, d) in graph.edges(data=True) if d["weight"] <= 0.5]

        pos = nx.drawing.nx_agraph.graphviz_layout(graph, prog="dot")

        # nodes
        nx.draw_networkx_nodes(graph, pos, node_size=15, ax=self.bn_ax)

        # edges
        # connectionstyle = [f"arc3,rad={r}" for r in it.accumulate([0.15] * 4)]
        connectionstyle = f"angle3,angleA=0, angleB=90"
        nx.draw_networkx_edges(
            graph,
            pos,
            edgelist=elarge,
            width=1,
            alpha=0.4,
            ax=self.bn_ax,
            connectionstyle=connectionstyle,
        )
        nx.draw_networkx_edges(
            graph,
            pos,
            edgelist=esmall,
            width=1,
            alpha=0.4,
            edge_color="b",
            style="dashed",
            ax=self.bn_ax,
            connectionstyle=connectionstyle,
        )

        # node labels
        nx.draw_networkx_labels(
            graph,
            pos,
            font_size=12,
            font_family="sans-serif",
            verticalalignment="bottom",
            ax=self.bn_ax,
        )

        # edge weight labels
        edge_labels: dict = nx.get_edge_attributes(graph, "weight")
        formatted_edge_labels = edge_labels.copy()

        for key in edge_labels.keys():
            formatted_edge_labels[key] = f"{edge_labels[key]:.2f}"

        nx.draw_networkx_edge_labels(
            graph,
            pos,
            formatted_edge_labels,
            font_size=12,
            connectionstyle=connectionstyle,
            ax=self.bn_ax,
        )
        self.fig.tight_layout(pad=3.0)
        self.draw()


class BayesianNetView(QSplitter):
    file_path_changed = Signal()

    def __init__(self, parent: QWidget | None = None) -> None:
        super().__init__(parent)
        self.depModel: CorrelationSQLProxyModel = None
        self._init_ui()

    def _init_ui(self):
        self.setOrientation(Qt.Orientation.Vertical)
        # BN Plot
        bnPlotLayout = QVBoxLayout()

        self.bn_canvas = BayesianNetCanvas()
        self.toolbar = NavigationToolbar2QT(self.bn_canvas, self)
        bnPlotLayout.addWidget(self.toolbar)
        bnPlotLayout.addWidget(self.bn_canvas)
        bnPlotWidget = QWidget()
        bnPlotWidget.setLayout(bnPlotLayout)
        self.addWidget(bnPlotWidget)

        # BN Visualization Settings
        settingsWidget = QWidget()
        settingsLayout = QHBoxLayout()

        # Checkbox for filtering by p-value
        self.filterByPValueCheckBox = QCheckBox(self.tr("Filter by P-Value"))
        self.filterByPValueCheckBox.stateChanged.connect(self.drawBN)
        settingsLayout.addWidget(self.filterByPValueCheckBox)

        # Dropdown for selecting BN Correlation Type

        self.correlationTypeComboBox = QComboBox()
        self.correlationTypeComboBox.addItems(
            ["Pearson", "Spearman", "Partial Correlation"]
        )
        self.correlationTypeComboBox.currentIndexChanged.connect(self.drawBN)
        self.correlationTypeComboBox.setCurrentIndex(1)
        self.correlationTypeComboBoxMapping = [
            PairTableSQLProxyModel.PearsonCorrRole,
            PairTableSQLProxyModel.SpearmanCorrRole,
            CorrelationSQLProxyModel.PartialCorrRole,
        ]
        self.pvalueTypeComboBoxMapping = [
            PairTableSQLProxyModel.PearsonPValueRole,
            PairTableSQLProxyModel.SpearmanPValueRole,
            CorrelationSQLProxyModel.PartialPValueRole,
        ]
        settingsLayout.addWidget(self.correlationTypeComboBox)

        # Slider Widget for adjusting BN Visualization Threshold
        self.threadSliderWidget = ExtendedSliderWidget()
        self.threadSliderWidget.setRange(0, 1)
        self.threadSliderWidget.setValue(0.3)
        self.threadSliderWidget.setTrackMovements(False)
        self.threadSliderWidget.valueChanged.connect(self.drawBN)
        settingsLayout.addWidget(self.threadSliderWidget)

        settingsWidget.setLayout(settingsLayout)
        self.addWidget(settingsWidget)

        # Explanation Widgets
        self.explanationWidget = QLabel(
            self.tr(
                "Here you can work with a visualization of the Bayesian Network.\n"
                "Using the dropdown above you can select the type of correlation to be used for drawing the Bayesian Network. "
                "Also you can adjust the threshold for edge visibility using the slider above and see how it affects the network.\n"
                "The buttons in the toolbar allow you to interact with the plot, including zooming and saving the plot in numerous formats."
            )
        )
        self.addWidget(self.explanationWidget)

    def setModels(self, depModel: CorrelationSQLProxyModel):
        self.depModel = depModel
        # self.depModel.filterInvalidated.connect(self.drawBN)
        self.depModel.dataChanged.connect(self.drawBN)

        self.drawBN()

    @Slot()
    def drawBN(self):
        if self.depModel is None or not self.isVisible():
            return
        print("BayesianNetView.drawBN")

        corr_matrix = tablemodel_to_dataframe(
            self.depModel,
            role=self.correlationTypeComboBoxMapping[
                self.correlationTypeComboBox.currentIndex()
            ],
        )
        pvalue_matrix = None

        if self.filterByPValueCheckBox.isChecked():
            pvalue_matrix = tablemodel_to_dataframe(
                self.depModel,
                role=self.pvalueTypeComboBoxMapping[
                    self.correlationTypeComboBox.currentIndex()
                ],
            )

        linkTable = tablemodel_to_dataframe(
            self.depModel, role=Qt.ItemDataRole.CheckStateRole
        )
        graph = GraphPreparation(
            corr_matrix,
            linkTable,
            self.threadSliderWidget.value(),
            pvalue_matrix=pvalue_matrix,
            pvalue_threshold=0.05,
        )

        graph.drop_cycle()
        # self.changeLinkTable()

        self.bn_canvas.update_plot(graph.renaming())
        # import pickle
        # pickle.dump(self.graph, open('graph.txt', 'w'))
        # print(self.graph.G.nodes())

        # nx.write_adjlist(self.graph.renaming(), 'graph.txt')

    def showEvent(self, event):
        v = super().showEvent(event)
        self.drawBN()
        return v



import numpy as np
import pandas as pd
import pingouin as pg


class CorrMatrix:
    def __init__(self, df):
        """
        Нахождение полной корреляции
        :param df:
        """
        self.df = df
        self.corr = self.df.corr(method="spearman")

    def getCorrMatrix(self, roundOrder=2):
        # return self.corr.round(roundOrder)
        return self.corr

    def updateTable(self, df):
        self.df = df
        self.corr = self.df.corr(method="spearman")


class PartCorrMatrix:
    def __init__(self, df: pd.DataFrame = None):
        """
        Нахождение частных корреляции
        :param parent:
        :param input_df:
        """
        self.df: pd.DataFrame = df
        self.corr = self.find_part_cor()
        self.corr = self.corr.astype("float")

    def find_part_cor(self):
        columns_all = self.df.columns
        P = pd.DataFrame(columns=self.df.columns, index=self.df.columns)
        for column_name1 in self.df.columns:
            for column_name2 in self.df.columns:
                if column_name1 == column_name2:
                    P.loc[column_name1, column_name2] = 1
                elif (
                    len(
                        pd.crosstab(self.df[column_name1], self.df[column_name2]).values
                    )
                    < 4
                ):
                    P.loc[column_name1, column_name2] = np.nan
                else:
                    columns_select = columns_all.drop(column_name1)
                    columns_select = columns_select.drop(column_name2)

                    try:
                        result = pg.partial_corr(
                            data=self.df,
                            x=column_name1,
                            y=column_name2,
                            covar=list(columns_select),
                            method="spearman",
                        )
                        P.loc[column_name1, column_name2] = result["r"].values[0]
                    except:
                        result = np.nan
        return P

    def getCorrMatrix(self):
        return self.corr



from PySide6.QtCore import (
    QModelIndex,
    QObject,
    QPersistentModelIndex,
    QSortFilterProxyModel,
    Qt,
    Signal,
    Slot,
)

from bn_modeller.models.checkable_sort_filter_proxy_model import (
    CheckableSortFilterProxyModel,
)


class RelationalSortFilterProxyModel(QSortFilterProxyModel):
    filterInvalidated = Signal()

    def __init__(self, parent: QObject = None):
        super().__init__(parent)
        self._filterModel: CheckableSortFilterProxyModel = None
        self._filterValueColumn: int = None
        self._filter_cache: dict

    def filterModel(self) -> CheckableSortFilterProxyModel:
        return self._filterModel

    def filterValueColumn(self) -> int:
        return self._filterValueColumn

    def setFilterModel(
        self, filterModel: CheckableSortFilterProxyModel, filterValueColumn: int
    ):
        if self._filterModel is not None:
            self._filterModel.dataChanged.disconnect(self.invalidateCache)
        self._filterModel = filterModel
        self._filterValueColumn = filterValueColumn
        self._filterModel.dataChanged.connect(self.invalidateCache)
        self.invalidateCache()

    def filterAcceptsRow(
        self, source_row: int, source_parent: QModelIndex | QPersistentModelIndex
    ):
        index = self.sourceModel().index(
            source_row, self.filterKeyColumn(), source_parent
        )
        return self._filter_cache.get(index.data(), False)

    @Slot(QModelIndex, QModelIndex, "QList<int>")
    def invalidateCache(
        self,
        topLeft: QModelIndex = None,
        bottomRight: QModelIndex = None,
        roles: list[int] = None,
    ):
        self._filter_cache = {}
        for rowIdx in range(self._filterModel.rowCount()):
            k = self._filterModel.data(
                self._filterModel.index(rowIdx, self._filterValueColumn)
            )
            v = self._filterModel.data(
                self._filterModel.index(rowIdx, self._filterValueColumn),
                role=Qt.ItemDataRole.CheckStateRole,
            )
            self._filter_cache[k] = v == Qt.CheckState.Checked
        self.invalidateFilter()
        self.filterInvalidated.emit()



import os
import shutil

import numpy as np
from PySide6.QtCore import QObject, QSettings, QStandardPaths, Qt, QUrl, Slot
from PySide6.QtGui import QDesktopServices, QDoubleValidator
from PySide6.QtSql import QSqlDatabase, QSqlQuery
from PySide6.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QFileDialog,
    QFormLayout,
    QGroupBox,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QRadioButton,
    QSizePolicy,
    QSpinBox,
    QVBoxLayout,
    QWizard,
    QWizardPage,
)

from bn_modeller.models.feature_sqltable_model import FeatureSqlTableModel
from bn_modeller.models.sample_sqltable_model import SampleSqlTableModel
from bn_modeller.utils.db_model_handler import add_values_from_csv
from bn_modeller.widgets.file_path_widget import FilePathWidget
from bn_modeller.widgets.separator_widget import QSeparator


class TableValueFixer(QObject):
    def __init__(self, parent):
        super().__init__(parent)
        self.cache = {}

    def askUserForNewValue(self, value: str) -> float:
        dialog = QDialog(parent=self.parent())
        dialog.setWindowTitle("Unexpected value")
        layout = QFormLayout()
        line_edit = QLineEdit()
        line_edit.setValidator(QDoubleValidator())  # only allow float inputs
        layout.addRow(
            QLabel(
                (
                    f"The program supports only numeric values for measurements, but found '{value}'.\n"
                    + f"Please enter a numeric value instead of '{value}'.\n"
                    + f"The value '{value}' will be replaced by NaN if you do not provide a valid number.\n"
                    + "All values in the dataset will be replaced by NaN or new value."
                )
            )
        )
        layout.addRow(QLabel(f"New Value instead of '{value}':"), line_edit)
        button_box = QDialogButtonBox(
            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
        )
        button_box.accepted.connect(dialog.accept)
        button_box.rejected.connect(dialog.reject)
        layout.addWidget(button_box)

        dialog.setLayout(layout)
        if dialog.exec() == QDialog.Accepted:
            new_value = line_edit.text()
            try:
                return float(new_value)
            except ValueError:
                print("Invalid input. Replacing with NaN.")
                return np.nan
        else:
            return np.nan

    def fixValue(self, value: str) -> float:
        if value not in self.cache:
            self.cache[value] = self.askUserForNewValue(value)
        return self.cache[value]


class DataSourceTemplatePage(QWizardPage):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setTitle(self.tr("Datasource Template"))
        self.path_edit: FilePathWidget
        self._init_ui()

    def _init_ui(self):
        main_layout = QVBoxLayout()

        explanation_label = QLabel(
            self.tr(
                "Please select a datasource template file. \n"
                "If you already have a datasource file with compatible format, you can skip this step.\n\n"
                "We provide two templates: \n"
                "1. A template for a CSV/Excel file with samples in rows.\n"
                "2. A template for a CSV/Excel file with samples in columns.\n\n"
                "Please select the appropriate template for your data."
            )
        )

        # Layout with buttons for selecting and saving the template file.
        hbox = QHBoxLayout()

        get_template_rows_button = QPushButton(
            self.tr("Get Template with samples in Rows")
        )
        get_template_rows_button.clicked.connect(self.on_get_template_rows_clicked)
        hbox.addWidget(get_template_rows_button)

        get_template_cols_button = QPushButton(
            self.tr("Get Template with samples in Columns")
        )
        get_template_cols_button.clicked.connect(self.on_get_template_cols_clicked)
        hbox.addWidget(get_template_cols_button)

        main_layout.addWidget(explanation_label)
        main_layout.addLayout(hbox)

        self.setLayout(main_layout)

    @Slot()
    def on_get_template_rows_clicked(self):
        # Code to handle the button click event
        fileName = QFileDialog.getSaveFileName(
            self,
            self.tr("Save Template"),
            None,
            "Comma-separated values File (*.csv);;Excel Workbook (*.xlsx)",
        )
        if len(fileName[0]) > 0:
            if fileName[0].endswith(".csv"):
                shutil.copyfile(
                    os.path.join(
                        os.path.dirname(__file__),
                        "..",
                        "resources",
                        "templates",
                        "datasource_rows.csv",
                    ),
                    fileName[0],
                )
            elif fileName[0].endswith(".xlsx"):
                shutil.copyfile(
                    os.path.join(
                        os.path.dirname(__file__),
                        "..",
                        "resources",
                        "templates",
                        "datasource_rows.xlsx",
                    ),
                    fileName[0],
                )
            QDesktopServices.openUrl(QUrl.fromLocalFile(os.path.dirname(fileName[0])))

    @Slot()
    def on_get_template_cols_clicked(self):
        fileName = QFileDialog.getSaveFileName(
            self,
            self.tr("Save Template"),
            None,
            "Comma-separated values File (*.csv);;Excel Workbook (*.xlsx)",
        )
        if len(fileName[0]) > 0:
            if fileName[0].endswith(".csv"):
                shutil.copyfile(
                    os.path.join(
                        os.path.dirname(__file__),
                        "..",
                        "resources",
                        "templates",
                        "datasource_cols.csv",
                    ),
                    fileName[0],
                )
            elif fileName[0].endswith(".xlsx"):
                shutil.copyfile(
                    os.path.join(
                        os.path.dirname(__file__),
                        "..",
                        "resources",
                        "templates",
                        "datasource_cols.xlsx",
                    ),
                    fileName[0],
                )
            QDesktopServices.openUrl(QUrl.fromLocalFile(os.path.dirname(fileName[0])))


class ProjectLocationPage(QWizardPage):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setTitle(self.tr("Select Data Source"))

        self.path_edit: FilePathWidget
        self._init_ui()

    def _init_ui(self):
        main_layout = QVBoxLayout()

        # createOrOpenRadioGroup
        groupBox = QGroupBox(self.tr("Create or Open"))

        self.radioOpen = QRadioButton(self.tr("Open existing"))
        self.radioOpen.setChecked(True)
        self.radioOpen.toggled.connect(self.changeFileMode)
        self.radioNew = QRadioButton(self.tr("Create New"))
        self.radioNew.toggled.connect(self.changeFileMode)

        vbox = QVBoxLayout()
        vbox.addWidget(self.radioOpen)
        vbox.addWidget(self.radioNew)
        # vbox.addStretch(1)
        groupBox.setLayout(vbox)

        main_layout.addWidget(groupBox)
        # File path row

        self.path_edit = FilePathWidget(
            self.tr("Select file"),
            self.tr("BNM Project File (*.sqlite)"),
            QSettings().value(
                "projectLoadWizard/lastProjectLocationDir",
                QStandardPaths.standardLocations(
                    QStandardPaths.StandardLocation.DocumentsLocation
                )[0],
            ),
            mode=FilePathWidget.FilePathMode.OpenFileName,
        )
        self.registerField(
            "ProjectLocationPage/projectLocation*", self.path_edit.path_edit
        )
        self.path_edit.file_path_changed.connect(self.saveLastFilePath)
        main_layout.addWidget(self.path_edit)

        self.setLayout(main_layout)

    def initializePage(self):
        res = super().initializePage()
        self.path_edit.file_path = QSettings().value(
            "projectLoadWizard/lastProjectLocation", ""
        )
        return res

    @Slot(bool)
    def changeFileMode(self, checked: bool):
        if checked:
            source = self.sender()
            if source == self.radioNew:
                self.path_edit.setMode(FilePathWidget.FilePathMode.SaveFileName)
                self.setFinalPage(False)
            elif source == self.radioOpen:
                self.path_edit.setMode(FilePathWidget.FilePathMode.OpenFileName)
                self.setFinalPage(True)

    @Slot(str)
    def saveLastFilePath(self, newFilePath: str):
        QSettings().setValue(
            "projectLoadWizard/lastProjectLocationDir", os.path.dirname(newFilePath)
        )
        QSettings().setValue("projectLoadWizard/lastProjectLocation", newFilePath)


class DataImportPage(QWizardPage):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setTitle(self.tr("Import data"))
        self._init_ui()

    def _init_ui(self):
        main_layout = QVBoxLayout()
        # File path row
        self.path_edit = FilePathWidget(
            self.tr("Select source file"),
            self.tr(
                "Comma-separated values File (*.csv);;Excel Workbook (*.xlsx *.xls)"
            ),
            QSettings().value(
                "DataImportPage/lastSourceLocationDir",
                QStandardPaths.standardLocations(
                    QStandardPaths.StandardLocation.DocumentsLocation
                )[0],
            ),
            mode=FilePathWidget.FilePathMode.OpenFileName,
        )
        self.path_edit.file_path_changed.connect(self.saveLastFilePath)
        main_layout.addWidget(self.path_edit)
        self.registerField("DataImportPage/csvPath*", self.path_edit.path_edit)

        self.setLayout(main_layout)

        # File settings group box
        groupBox = QGroupBox(self.tr("Source file format"))
        vbox = QVBoxLayout()

        # Add a label to explain the format options
        orientation_expl_label = QLabel(
            self.tr(
                "Select how samples are organized in your file. If samples are in rows, each row represents a sample. If samples are in columns, each column represents a sample."
            )
        )
        orientation_expl_label.setWordWrap(True)
        orientation_expl_label.setSizePolicy(
            QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum
        )
        vbox.addWidget(orientation_expl_label)

        self.radioSampleInRow = QRadioButton(self.tr("Samples in rows"))
        self.radioSampleInRow.setChecked(True)
        self.radioSampleInColumn = QRadioButton(self.tr("Samples in columns"))

        vbox.addWidget(self.radioSampleInRow)
        vbox.addWidget(self.radioSampleInColumn)
        vbox.addWidget(QSeparator())

        # Add a new HBoxLayout with two spinboxes and labels for each one
        hbox = QHBoxLayout()
        self.skip_rows_spinbox = QSpinBox()
        self.skip_rows_spinbox.setRange(0, 100)  # Set the range to 100
        hbox.addWidget(QLabel(self.tr("Skip rows: ")))
        hbox.addWidget(self.skip_rows_spinbox)

        self.skip_cols_spinbox = QSpinBox()
        self.skip_cols_spinbox.setRange(0, 100)  # Set the range to 100
        hbox.addWidget(QLabel(self.tr("Skip columns: ")))
        hbox.addWidget(self.skip_cols_spinbox)
        vbox.addLayout(hbox)

        # Add a label to explain the purpose of the skip rows and columns fields
        skip_expl_label = QLabel(
            self.tr(
                "If you are using a custom format, you can specify the number of rows and columns to skip to make the dataset compatible with the program."
            )
        )
        skip_expl_label.setWordWrap(True)
        vbox.addWidget(skip_expl_label)

        groupBox.setLayout(vbox)
        self.registerField("DataImportPage/isSampleInRows", self.radioSampleInRow)
        self.registerField("DataImportPage/skipRows", self.skip_rows_spinbox)
        self.registerField("DataImportPage/skipColumns", self.skip_cols_spinbox)
        main_layout.addWidget(groupBox)

    @Slot(str)
    def saveLastFilePath(self, newFilePath: str):
        QSettings().setValue(
            "DataImportPage/lastSourceLocationDir", os.path.dirname(newFilePath)
        )


class ProjectLoadWizard(QWizard):
    def __init__(self, pre_done_handler=None, parent=None):
        super().__init__(parent)
        # self.setWindowTitle(self.tr("Open project"))
        self._pre_done_handler = pre_done_handler

        self.source_page = ProjectLocationPage()
        self.sourcePageId = self.addPage(self.source_page)

        self.datasourceTemplatePage = DataSourceTemplatePage()
        self.datasourceTemplatePageId = self.addPage(self.datasourceTemplatePage)

        self.importDataPage = DataImportPage()
        self.importDataPageId = self.addPage(self.importDataPage)

        self.button(QWizard.FinishButton).clicked.connect(self.close)

    def get_title(self):
        return self.tr("Open project")

    def get_project_path(self) -> str:
        return self.source_page.path_edit.file_path

    def nextId(self):
        if self.currentPage() == self.source_page:
            if self.source_page.radioOpen.isChecked():
                return -1
            else:
                return self.datasourceTemplatePageId
        return super().nextId()

    def createDb(self):
        query = QSqlQuery()
        query.exec("PRAGMA page_size = 4096;")
        query.exec("PRAGMA cache_size = 16384;")
        query.exec("PRAGMA temp_store = MEMORY;")
        query.exec("PRAGMA journal_mode = PERSIST;")
        query.exec("PRAGMA locking_mode = EXCLUSIVE;")
        # WARNING: IT IS NOT SAFE. It can cause a DB damage in case of a bad termination.
        query.exec("PRAGMA synchronous = OFF;")
        self.loadModelsFromDb()

        try:
            valueFixer = TableValueFixer(self)
            add_values_from_csv(
                self.field("DataImportPage/csvPath"),
                not self.field("DataImportPage/isSampleInRows"),
                self.featureSqlTableModel,
                self.sampleSqlTableModel,
                skip_rows=self.field("DataImportPage/skipRows"),
                skip_cols=self.field("DataImportPage/skipColumns"),
                value_fixer_callback=valueFixer.fixValue,
            )
        except Exception as e:
            # TODO: handle the exception, ask the user to retry or quit
            print(e)

    def loadModelsFromDb(self):
        self.featureSqlTableModel = FeatureSqlTableModel(db=self._db)
        self.sampleSqlTableModel = SampleSqlTableModel(db=self._db)

    def connectDb(self):
        self._db = None
        ProjectLoadWizard.closeDb()
        self._db = ProjectLoadWizard.openDb(self.source_page.path_edit.file_path)

    def done(self, result):
        if self._pre_done_handler is not None:
            self._pre_done_handler(self, result)
        if result:
            if self.source_page.radioOpen.isChecked():
                self.connectDb()
                self.loadModelsFromDb()
            else:
                if os.path.exists(self.source_page.path_edit.file_path):
                    # TODO: ask user to delete the file
                    os.remove(self.source_page.path_edit.file_path)
                self.connectDb()
                self.createDb()

        return super().done(result)

    @staticmethod
    def closeDb():
        for conn_name in QSqlDatabase.connectionNames():
            db = QSqlDatabase.database(conn_name)
            db.commit()
            db.close()
            QSqlDatabase.removeDatabase(conn_name)

    @staticmethod
    def openDb(path: str) -> QSqlDatabase:
        db = QSqlDatabase.addDatabase("QSQLITE")
        db.setDatabaseName(path)
        db.open()
        return db



from PySide6.QtCore import QAbstractItemModel, Qt, Signal, Slot
from PySide6.QtWidgets import QHBoxLayout, QTableView, QWidget

from bn_modeller.models.sample_sqltable_model import SampleSqlTableModel
from bn_modeller.widgets.all_samples_view import AllSamplesView


class RegrPlotPageWidget(QWidget):
    def __init__(self, parent: QWidget | None = None, f=Qt.WindowType()):
        super().__init__(parent, f)
        self._init_ui()

    def _init_ui(self):
        self.mainLayout = QHBoxLayout(self)



import os

from PySide6.QtCore import QCoreApplication, Qt, Signal, Slot
from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QMainWindow, QStyle, QToolBar, QWidget


class BaseWindow(QMainWindow):
    def __init__(
        self, title: str, parent: QWidget | None = None, flags=Qt.WindowType()
    ) -> None:
        super(BaseWindow, self).__init__(parent, flags)
        self.caption = title
        # self.setup_toolbar()
        my_icon = QIcon()
        my_icon.addFile(
            os.path.join(os.path.dirname(__file__), "..", "resources\\icon.ico")
        )
        self.setWindowIcon(my_icon)
        self.set_central_title(title)
        self.statusBar().showMessage("Ready")

    def set_central_title(self, title):
        self.setWindowTitle(title)

    # def setup_toolbar(self):
    #     self._toolbar = QToolBar(self.tr('Main ToolBar'))

    #     back_action = QAction(self.style().standardIcon(
    #         QStyle.StandardPixmap.SP_ArrowBack), '&Back', self)
    #     back_action.setStatusTip(self.tr('Go Back'))
    #     back_action.triggered.connect(self.go_back_clicked)

    #     self.addToolBar(self._toolbar)
    #     self._toolbar.addAction(back_action)

    def getMainToolBar(self) -> QToolBar:
        return self._toolbar

    @Slot()
    def go_back_clicked(self):
        raise NotImplementedError()

    @Slot()
    def home_clicked(self):
        raise NotImplementedError()

    @Slot()
    def exit_clicked(self):
        return

    @Slot()
    def close_app(self):
        QCoreApplication.instance().quit()



import copy
from textwrap import wrap

import matplotlib.pyplot as plt
import networkx as nx
import numpy as np
import pandas as pd
import seaborn as sns
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
from PySide6 import QtCore, QtGui, QtWidgets

from bn_modeller.bayesian_nets import CorrMatrix, PartCorrMatrix
from bn_modeller.bayesian_nets.graph_preparation import GraphPreparation
from bn_modeller.bayesian_nets.pyBansheeCalculation import BansheeCalc
from bn_modeller.bayesian_nets.utils import get_outliers_cooks


def rescale_feature(input_vector, lower, upper):
    xmin, xmax = np.min(input_vector), np.max(
        input_vector
    )  # get max and min from input array

    input_vector = [(x - xmin) / (xmax - xmin) for x in input_vector]

    input_vector = [lower + (upper - lower) * x for x in input_vector]
    return input_vector


def test(y_pred, y_true):
    columns_list = y_true.columns
    result_list = {}
    for num, column_name in enumerate(columns_list):
        # k = y_true.iloc[:, num].dropna()
        # upper, lower = k.max(), k.min()
        df_result = pd.DataFrame(
            data={"pred": y_pred[:, num], "true": y_true.iloc[:, num]},
            index=y_true.index,
        )
        result_list[column_name] = df_result
    return result_list


def find_outliers(vector: pd.Series):
    outliers_index = vector.loc[
        (vector > vector.std() * 3 + vector.mean())
        | (vector < vector.mean() - vector.std() * 3)
    ].index
    return list(outliers_index)


class SubplotWindow(QtWidgets.QMainWindow):

    def __init__(self, parent=None, data=None):
        super(SubplotWindow, self).__init__(parent)

        self._main = QtWidgets.QWidget()
        self.setCentralWidget(self._main)
        layout = QtWidgets.QHBoxLayout(self._main)

        layout_widget = {}
        layoutVert = {}
        for i in data:
            layout_widget[i] = QtWidgets.QWidget()
            layoutVert[i] = QtWidgets.QVBoxLayout(self._main)
            layout_widget[i].setLayout(layoutVert[i])
            fig = self.initFigure(data[i], i)
            fig.tight_layout()
            canvas = FigureCanvas(fig)

            layoutVert[i].addWidget(NavigationToolbar(canvas, self))
            layoutVert[i].addWidget(canvas)

            layout.addWidget(layout_widget[i])

        # self.resize(2000, 1000)

        # layout_widget = QtWidgets.QWidget()
        # layoutVert = QtWidgets.QVBoxLayout()
        # self.setCentralWidget(layout_widget)
        #
        # # layout_widget = {}
        # # layoutVert = {}
        # m = len(data)
        # if m <= 4:
        #     layout_widget_H = QtWidgets.QWidget()
        #     layoutHor = QtWidgets.QHBoxLayout(layout_widget_H)
        #     for i in data:
        #         widgTmp = QtWidgets.QWidget()
        #         layoutUn = QtWidgets.QVBoxLayout(widgTmp)
        #         fig = self.initFigure(data[i], i)
        #         fig.tight_layout()
        #         canvas = FigureCanvas(fig)
        #         layoutUn.addWidget(NavigationToolbar(canvas, self))
        #         layoutUn.addWidget(canvas)
        #         layoutHor.addWidget(widgTmp)
        #
        #     layout_widget.setLayout(layoutHor)
        #     layoutVert.addWidget(layout_widget)
        # # else:
        # #     for i in range(m//4):
        # #         k_start = i * 4
        # #         for k in range(k_start, k_start+4):
        # #             if k > len(data):
        # #                 break
        # #             i = data.keys()[k]
        # #             layout_widget[i] = QtWidgets.QWidget()
        # #             layoutVert[i] = QtWidgets.QVBoxLayout(self._main)
        # #             layout_widget[i].setLayout(layoutVert[i])
        # #             fig = self.initFigure(data[i], i)
        # #             fig.tight_layout()
        # #             canvas = FigureCanvas(fig)
        # #
        # #             layoutVert[i].addWidget(NavigationToolbar(canvas, self))
        # #             layoutVert[i].addWidget(canvas)

    def initFigure(self, df, name):
        # plt.rcParams.update({'figure.autolayout': True})
        name = "\n".join(wrap(name, 30))
        fig = plt.figure(layout="tight")
        out = find_outliers(df["true"] - df["pred"])
        df_drop_out = df[~df.index.isin(out)]

        df_drop_out = df_drop_out.dropna()

        # sns.regplot(x=df_drop_out['true'], y=df_drop_out['pred'], color='tab:blue',
        #             scatter=False, line_kws={'linewidth': 1})

        xseq = np.linspace(df["true"].min() - 5, df["true"].max() + 5, num=100)
        plt.scatter(df_drop_out["true"], df_drop_out["pred"])
        if len(df_drop_out) > 1:
            b, a = np.polyfit(df_drop_out["true"], df_drop_out["pred"], deg=1)
            plt.plot(xseq, a + b * xseq, lw=1)

        # ci = 1.96 * np.std(a + b * xseq) / np.sqrt(len(xseq))
        # plt.fill_between(xseq, (a + b * xseq - ci), (a + b * xseq + ci), color='b', alpha=.1)

        r2 = df["true"].corr(df["pred"])
        r2_drop_out = df_drop_out["true"].corr(df_drop_out["pred"])

        if len(out):
            k = df.loc[out]
            # add linear regression
            plt.scatter(k["true"], k["pred"], color="r")

        plt.xlabel(f"Observed, {name}")
        plt.ylabel(f"Predicted, {name}")
        plt.grid()
        plt.title(rf"$R$ {round(r2, 3)}$\rightarrow${round(r2_drop_out, 3)}")
        return fig


class SubplotGraph(SubplotWindow):
    def initFigure(self, G, name):
        fig = plt.figure(figsize=(20, 20))
        elarge = [(u, v) for (u, v, d) in G.edges(data=True) if d["weight"] > 0.5]
        esmall = [(u, v) for (u, v, d) in G.edges(data=True) if d["weight"] <= 0.5]

        # pos = nx.spring_layout(G, seed=10, iterations=200)  # positions for all nodes - seed for reproducibility
        # pos = nx.fruchterman_reingold_layout(G)  # positions for all nodes - seed for reproducibility
        # pos = nx.circular_layout(G)
        # pos = nx.spectral_layout(G)
        # pos = nx.multipartite_layout(G)
        # pos = nx.planar_layout(G)
        # pos = nx.drawing.nx_agraph.graphviz_layout(G)
        pos = nx.drawing.nx_agraph.graphviz_layout(G, prog="dot")

        # nodes
        nx.draw_networkx_nodes(G, pos, node_size=5)

        # edges
        nx.draw_networkx_edges(G, pos, edgelist=elarge, width=1, alpha=0.4)
        nx.draw_networkx_edges(
            G, pos, edgelist=esmall, width=1, alpha=0.4, edge_color="b", style="dashed"
        )

        # node labels
        nx.draw_networkx_labels(
            G, pos, font_size=8, font_family="sans-serif", verticalalignment="bottom"
        )

        # edge weight labels
        edge_labels = nx.get_edge_attributes(G, "weight")
        nx.draw_networkx_edge_labels(G, pos, edge_labels, font_size=8)

        ax = plt.gca()
        ax.margins(0.08)
        plt.axis("off")
        plt.tight_layout()
        plt.savefig("imagenet_layout.eps")
        return fig


class PlotWindows(QtWidgets.QMainWindow):

    def __init__(self, parent=None, data=None, title="Matrix"):
        super(PlotWindows, self).__init__(parent)

        self.setWindowTitle(title)

        self.main_widget = QtWidgets.QWidget(self)
        self.resize(2000, 1000)

        self.data = data
        self.fig = Figure()

        self.initFigure()

        self.canvas = FigureCanvas(self.fig)

        self.canvas.setSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
        )
        self.canvas.updateGeometry()
        # self.label = QtWidgets.QLabel("A plot:")
        toolbar = NavigationToolbar(self.canvas, self)

        self.layout = QtWidgets.QGridLayout(self.main_widget)
        self.layout.addWidget(toolbar)
        # self.layout.addWidget(self.label)
        self.layout.addWidget(self.canvas)

        self.setCentralWidget(self.main_widget)
        self.show()
        self.fig.tight_layout()

    def initFigure(self):
        ax = self.fig.add_subplot(111)
        # discards the old graph
        ax.clear()

        column_name = ["\n".join(wrap(text, 30)) for text in self.data.columns.values]
        print(self.data)
        im = sns.heatmap(
            self.data,
            xticklabels=column_name,
            yticklabels=column_name,
            annot=True,
            ax=ax,
            annot_kws={"size": 8},
        )

        ax.set_xticklabels(column_name, rotation=45, ha="right", fontsize=8)
        ax.set_yticklabels(column_name, rotation=0, ha="right", fontsize=8)


class PlotInferenceResult(QtWidgets.QMainWindow):
    def __init__(self, parent=None, data=None, name=None, title="Matrix"):
        super(PlotInferenceResult, self).__init__(parent)

        self.setWindowTitle(title)

        self.main_widget = QtWidgets.QWidget(self)
        self.resize(600, 400)

        self.data = data
        self.fig = Figure()

        self.initFigure(self.data, name)

        self.canvas = FigureCanvas(self.fig)

        self.canvas.setSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
        )
        self.canvas.updateGeometry()
        # self.label = QtWidgets.QLabel("A plot:")
        toolbar = NavigationToolbar(self.canvas, self)

        self.layout = QtWidgets.QGridLayout(self.main_widget)
        self.layout.addWidget(toolbar)
        # self.layout.addWidget(self.label)
        self.layout.addWidget(self.canvas)

        self.setCentralWidget(self.main_widget)
        self.show()
        self.fig.tight_layout()

    def initFigure(self, df, name):
        # plt.rcParams.update({'figure.autolayout': True})
        name = "\n".join(wrap(name, 30))
        ax = self.fig.add_subplot(111)
        ax.grid()
        # out = find_outliers(df['true']-df['pred'])

        out = get_outliers_cooks(df.dropna())

        df_drop_out = df[~df.index.isin(out)]

        df_drop_out = df_drop_out.dropna()

        # sns.regplot(x=df_drop_out['true'], y=df_drop_out['pred'], color='tab:blue',
        #             scatter=False, line_kws={'linewidth': 1})

        xseq = np.linspace(df["pred"].min() - 5, df["pred"].max() + 5, num=100)
        ax.scatter(df_drop_out["pred"], df_drop_out["true"])
        if len(df_drop_out) > 1:
            b, a = np.polyfit(df_drop_out["pred"], df_drop_out["true"], deg=1)
            ax.plot(xseq, a + b * xseq, lw=1)

        # ci = 1.96 * np.std(a + b * xseq) / np.sqrt(len(xseq))
        # plt.fill_between(xseq, (a + b * xseq - ci), (a + b * xseq + ci), color='b', alpha=.1)

        r2 = df["true"].corr(df["pred"])
        r2_drop_out = df_drop_out["true"].corr(df_drop_out["pred"])

        if len(out):
            k = df.loc[out]
            # add linear regression
            ax.scatter(k["pred"], k["true"], color="r")

        ax.set_xlabel(f"Predicted, {name}")
        ax.set_ylabel(f"Observed, {name}")

        ax.set_title(rf"$R$ {round(r2, 3)}$\rightarrow${round(r2_drop_out, 3)}")
        # return fig


class ButtonWindow(QtWidgets.QWidget):
    def __init__(self, parent=None, input_df=None, linkTable=None, len_input=None):
        super(ButtonWindow, self).__init__(parent)
        self.input_df: pd.DataFrame = input_df
        self.setFixedSize(400, 500)
        self.move(0, 0)

        self.corr_button = QtWidgets.QPushButton("Full Spearman's Correlation", self)
        self.corr_button.clicked.connect(self.on_pushButton_clicked)

        self.part_corr_button = QtWidgets.QPushButton(
            "Partial Spearman's Correlation", self
        )
        self.part_corr_button.clicked.connect(self.on_part_corr_button_clicked)

        self.acycle_button = QtWidgets.QPushButton("Searching acyclic graph", self)
        self.acycle_button.clicked.connect(self.on_acycle_graph)

        self.rankCorrButton = QtWidgets.QPushButton(
            "Reconstruction of the correlation (Banshee)", self
        )
        self.rankCorrButton.clicked.connect(self.onRankCorrBanshee)
        self.rankCorrButton.setEnabled(False)

        self.calcInferenceButton = QtWidgets.QPushButton(
            "Calculating Bayesian model inference (Banshee)", self
        )
        self.calcInferenceButton.clicked.connect(self.onInferenceButton)
        self.calcInferenceButton.setEnabled(False)

        self.thresholdEdit = QtWidgets.QLineEdit(self, placeholderText="0.0")
        validator = QtGui.QDoubleValidator()  # Создание валидатора.
        validator.setRange(0.0, 1.0, 2)  # Установка диапазона значений.
        validator.setLocale(QtCore.QLocale("en_US"))
        # Установка валидатора для поля ввода
        self.thresholdEdit.setValidator(validator)
        self.thresholdEdit.textChanged[str].connect(self.onChanged)

        lay = QtWidgets.QGridLayout()
        lay.setSpacing(0)
        lay.setContentsMargins(0, -1, 0, -1)

        lay.addWidget(QtWidgets.QLabel("Threshold"), 1, 0)
        lay.addWidget(self.thresholdEdit, 1, 1)
        # lay.addWidget(self.formLayoutWidget)
        lay.addWidget(self.corr_button, 2, 0, 1, 2)
        lay.addWidget(self.part_corr_button, 3, 0, 1, 2)

        lay.addWidget(self.acycle_button, 4, 0, 1, 2)
        lay.addWidget(self.rankCorrButton, 5, 0, 1, 2)
        lay.addWidget(self.calcInferenceButton, 6, 0, 1, 2)
        self.setLayout(lay)

        self.dialogs = list()
        self.corr_matrix = CorrMatrix(self.input_df).getCorrMatrix()
        self.partCorrMatrix = PartCorrMatrix(self.input_df).getCorrMatrix()

        self.linkTable = linkTable
        self.len_input = len_input

        self.y_true = self.input_df.iloc[:, self.len_input :]

        # self.setGeometry(300, 300, 350, 300)

        self.thresholdValue = 0.0

        self.updLinkTable = None
        self.df_predicted = pd.DataFrame()
        self.error_dialog = QtWidgets.QErrorMessage()

    def onChanged(self, text):
        self.thresholdValue = float(text)

    @QtCore.Slot()
    def on_pushButton_clicked(self):
        dialog = PlotWindows(self, self.corr_matrix)
        self.dialogs.append(dialog)
        dialog.show()

    @QtCore.Slot()
    def on_part_corr_button_clicked(self):
        if len(self.partCorrMatrix.dropna()) < 1:
            self.error_dialog.showMessage("Error of the correlation matrix")
        else:
            dialog = PlotWindows(self, self.partCorrMatrix)
            self.dialogs.append(dialog)
            dialog.show()

    def on_acycle_graph(self):
        self.graph = GraphPreparation(
            self.corr_matrix, self.linkTable, self.thresholdValue
        )

        # удалить циклы в графе
        G_before = copy.deepcopy(self.graph.renaming())

        self.graph.drop_cycle()
        self.changeLinkTable()

        d = {"Before": G_before, "After": self.graph.renaming()}
        # import pickle
        # pickle.dump(self.graph, open('graph.txt', 'w'))
        # print(self.graph.G.nodes())

        nx.write_adjlist(self.graph.renaming(), "graph.txt")
        dialog = SubplotGraph(data=d)
        self.dialogs.append(dialog)
        dialog.show()
        self.rankCorrButton.setEnabled(True)

    def onRankCorrBanshee(self):
        self.banshee = BansheeCalc(
            self.graph.getNodeList(), self.graph.getEdgeList(), self.input_df
        )
        self.R = self.banshee.getRankCorr()
        if self.R is None:
            self.error_dialog.showMessage(
                "Error in the reconstruction of the correlation matrix"
            )
        else:
            self.banshee.saveGraph()

        column_name = self.input_df.columns
        dialog = PlotWindows(
            self, data=pd.DataFrame(self.R, columns=column_name, index=column_name)
        )
        self.dialogs.append(dialog)
        dialog.show()
        plt.tight_layout()
        self.calcInferenceButton.setEnabled(True)

    def onInferenceButton(self):
        y_predict = self.banshee.getInference(self.len_input)

        self.columnsForPredict = self.input_df.columns[self.len_input :]
        self.columnsFeatures = self.input_df.columns[: self.len_input]

        pred_column = ["Predicted " + i for i in self.columnsForPredict]
        self.df_predicted = pd.DataFrame(
            y_predict, index=self.input_df.index, columns=pred_column
        )
        df = self.input_df.join(self.df_predicted)

        df_tmp = df.copy(deep=True)
        df_tmp = df_tmp.dropna(subset=self.columnsFeatures)

        for col_name in self.columnsForPredict:
            lower, upper = self.input_df[col_name].min(), self.input_df[col_name].max()
            df_tmp.loc[:, "Predicted " + col_name] = rescale_feature(
                df_tmp.loc[:, "Predicted " + col_name], lower, upper
            )

        df = self.input_df.join(df_tmp[pred_column])
        self.df_predicted = df[pred_column]

        df.to_csv("data/result.csv")

        df = df.dropna(subset=self.columnsFeatures)

        # dictPredict = test(y_predict, self.y_true)
        dictPredict = test(df[pred_column].values, df[self.columnsForPredict])
        # dialog = SubplotWindow(data=dictPredict)
        for i in dictPredict:
            dialog = PlotInferenceResult(data=dictPredict[i], name=i)
            self.dialogs.append(dialog)
            dialog.show()
            plt.tight_layout()

    def updateDataFrame(self, input_df):
        self.input_df = input_df

    def updateLinkTable(self, linkTable):
        self.linkTable = linkTable
        self.updLinkTable = None

    def updateLenInputFeature(self, lenInput):
        self.len_input = lenInput

    def updateMatrix(self):
        self.corr_matrix = CorrMatrix(self.input_df).getCorrMatrix()
        # self.partCorrMatrix = PartCorrMatrix(self.input_df).getCorrMatrix()
        self.y_true = self.input_df.iloc[:, self.len_input :]

    def getThresholdVal(self):
        return self.thresholdValue

    def changeLinkTable(self):
        newLinkTab = pd.DataFrame(
            columns=self.input_df.columns, index=self.input_df.columns
        )

        for i in self.graph.renaming().edges(data=True):
            newLinkTab.loc[i[0], i[1]] = 1
        newLinkTab = newLinkTab.fillna(0)
        self.updLinkTable = newLinkTab

    def getNewLinkTab(self):
        return self.updLinkTable

    def getPrediction(self):
        return self.df_predicted



import numpy as np
import pandas as pd
from PySide6 import QtCore, QtWidgets


class LinkTabWindow(QtWidgets.QWidget):
    def __init__(self, parent=None, input_df=None, state=None):
        """
        Для матрицы связей
        :param parent:
        :param input_df:
        """
        super(LinkTabWindow, self).__init__(parent)

        self.widget_page = QtWidgets.QWidget()
        self.input_df = input_df
        self.columnNames = list(input_df.columns)
        self.result_df = pd.DataFrame(columns=self.columnNames, index=self.columnNames)

        self.columnNames.insert(0, "Select ALL")
        self.tableWidget = QtWidgets.QTableWidget(
            len(self.columnNames), len(self.columnNames)
        )

        self.state = (
            self.updateState(state)
            if state is not None and (len(state) + 1 == len(self.columnNames))
            else self.initState()
        )
        for i in range(self.tableWidget.rowCount()):
            for j in range(self.tableWidget.columnCount()):
                item = QtWidgets.QTableWidgetItem()
                item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)
                # item.setCheckState(QtCore.Qt.Unchecked)
                item.setCheckState(self.state[i][j])
                self.tableWidget.setItem(i, j, item)

        self.lay = QtWidgets.QVBoxLayout(self)

    def select_all_clicked_by_columns(self, row, column):
        if row != 0:
            return 0
        state = self.tableWidget.item(0, column).checkState()
        for i in range(self.tableWidget.rowCount()):
            item = self.tableWidget.item(i, column)
            item.setCheckState(QtCore.Qt.Checked if state else QtCore.Qt.Unchecked)

    def select_all_clicked_by_rows(self, row, column):
        if column != 0:
            return 0
        state = self.tableWidget.item(row, 0).checkState()
        for i in range(self.tableWidget.columnCount()):
            item = self.tableWidget.item(row, i)
            item.setCheckState(QtCore.Qt.Checked if state else QtCore.Qt.Unchecked)

    @QtCore.Slot()
    def save_clicked(self):
        items = []
        for i in range(1, self.tableWidget.rowCount()):
            it = []
            for j in range(1, self.tableWidget.columnCount()):
                item = self.tableWidget.item(i, j)
                it.append(item.checkState())
            items.append(it)

        self.result_df.loc[:, :] = items
        self.result_df = self.result_df.replace({2: 1})
        if len(self.result_df) > 1:
            self.result_df.to_excel("data/link_table.xlsx", engine="openpyxl")

    def getDataFrame(self) -> pd.DataFrame:
        return self.result_df

    def updateInput(self, input_df, state=None):
        self.input_df = input_df

        self.columnNames = input_df.columns
        self.result_df = pd.DataFrame(columns=self.columnNames, index=self.columnNames)

        self.columnNames = self.columnNames.insert(0, "Select ALL")
        self.tableWidget = QtWidgets.QTableWidget(
            len(self.columnNames), len(self.columnNames)
        )

        self.tableWidget.setHorizontalHeaderLabels(self.columnNames)
        self.tableWidget.setVerticalHeaderLabels(self.columnNames)
        if state is not None:
            if (type(state[0][0]) == np.int64) & (
                len(state) + 1 == self.tableWidget.rowCount()
            ):
                self.state = self.updateState(state)
            else:
                self.state = self.initState()
        else:
            self.state = self.initState()

        for i in range(self.tableWidget.rowCount()):
            for j in range(self.tableWidget.columnCount()):
                item = QtWidgets.QTableWidgetItem()
                item.setFlags(item.flags() | QtCore.Qt.ItemIsUserCheckable)
                # item.setCheckState(QtCore.Qt.Unchecked)
                item.setCheckState(self.state[i][j])
                self.tableWidget.setItem(i, j, item)

        self.tableWidget.cellChanged.connect(self.select_all_clicked_by_columns)
        self.tableWidget.cellChanged.connect(self.select_all_clicked_by_rows)

        self.lay.addWidget(self.tableWidget)

    def removeTableWidget(self):
        self.tableWidget.deleteLater()

    def initState(self):
        return [
            [QtCore.Qt.Unchecked] * self.tableWidget.columnCount()
        ] * self.tableWidget.rowCount()

    def updateState(self, input_vector):
        new_vector = [
            [QtCore.Qt.Unchecked] * self.tableWidget.columnCount()
            for _ in range(self.tableWidget.rowCount())
        ]

        for i in range(0, len(input_vector)):
            for j in range(0, len(input_vector[0])):
                new_vector[i + 1][j + 1] = (
                    QtCore.Qt.Checked if input_vector[i][j] > 0 else QtCore.Qt.Unchecked
                )
        return new_vector

    def getColumnNames(self):
        return self.columnNames[1:]

    def saveState(self):
        self.state = []
        for i in range(self.tableWidget.rowCount()):
            it = []
            for j in range(self.tableWidget.columnCount()):
                item = self.tableWidget.item(i, j)
                it.append(item.checkState())
            self.state.append(it)



import ctypes
import os
import sys

from PySide6.QtCore import QCommandLineOption, QCommandLineParser, QCoreApplication, Qt
from PySide6.QtWidgets import QApplication

from bn_modeller import __version__
from bn_modeller.windows.main_window import MainWindow


def create_application(argv: list[str]) -> QApplication:
    QCoreApplication.setOrganizationName("Digiratory")
    QCoreApplication.setOrganizationDomain("digiratory.ru")
    QCoreApplication.setApplicationName("bn_modeller")
    QCoreApplication.setApplicationVersion(__version__)
    bn_modeller_appid = "digiratory.bn_modeller.{__version__}"
    if os.name == "nt":
        ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(bn_modeller_appid)
    if "no-gui" in argv:
        app = QCoreApplication(argv)
    else:
        app = QApplication(argv)
    return app


def manage_cli_args(app: QCoreApplication) -> QCommandLineParser:
    cl_parser: QCommandLineParser = QCommandLineParser()
    cl_parser.setApplicationDescription("Bayesian Network Modeller")
    cl_parser.addHelpOption()
    cl_parser.addVersionOption()

    no_gui = QCommandLineOption("nogui", "Launch in headless mode")
    cl_parser.process(app)

    return cl_parser


def main():
    app = create_application(sys.argv)
    cl_parser = manage_cli_args(app)

    if isinstance(app, QApplication):
        main_window = MainWindow()
        main_window.show()
    else:
        raise ValueError("Headless mode is not supported")
    sys.exit(app.exec())


if __name__ == "__main__":
    main()



import networkx as nx


def melt_matrix(matrix):
    """
    Преобразование из матрицы в таблицу
    :param matrix:
    :return:
    """
    matrix = matrix.stack().reset_index()
    matrix.columns = ["row", "column", "value"]
    # result = matrix[~(matrix['row'] == matrix['column'])]
    result = matrix
    return result


class GraphPreparation:
    def __init__(
        self,
        corr_matrix,
        table_connection,
        threshold,
        pvalue_matrix=None,
        pvalue_threshold=0.05,
    ):
        if pvalue_matrix is not None:
            assert (
                corr_matrix.shape == pvalue_matrix.shape
            ), "Correlation matrix and p-value matrix must have the same shape"

        self.threshold = threshold
        self.corr_matrix = corr_matrix
        self.table_connection = table_connection
        self.pvalue_matrix = pvalue_matrix
        self.code_columns = {num: i for i, num in enumerate(table_connection)}

        if pvalue_matrix is not None:
            self.corr_matrix = self.corr_matrix.mul(
                pvalue_matrix.map(lambda x: x < pvalue_threshold)
            )

        weight_matrix = self.corr_matrix * self.table_connection.map(
            lambda x: bool(x.value)
        )

        weight_matrix = weight_matrix.rename(self.code_columns, axis=0)
        weight_matrix = weight_matrix.rename(self.code_columns, axis=1)

        # подготовить матрицу к таблице
        self.N = melt_matrix(weight_matrix)
        self.N = self.N[(abs(self.N["value"]) > threshold)]
        self.N[["row", "column"]] = self.N[["row", "column"]].astype(int)

        # Создание взвешенного графа
        self.G = nx.DiGraph()

        e = [tuple([int(i[0]), int(i[1]), i[2]]) for i in self.N.values]

        self.G.add_weighted_edges_from(e)

    def drop_cycle(self):
        while True:
            try:
                cycle_list = nx.find_cycle(self.G)
            except:
                break

            min_nodes = ()
            min_val = 10

            for i in cycle_list:
                weight = self.G.get_edge_data(*i)["weight"]
                if weight < min_val:
                    min_val = weight
                    min_nodes = i

            self.G.remove_edge(*min_nodes)

    def getNodeList(self):
        return self.G.nodes

    def getEdgeList(self):
        return self.G.edges

    def getGraph(self):
        return self.G

    def renaming(self):
        k = {val: key for key, val in self.code_columns.items()}

        return nx.relabel_nodes(self.G, k)
